/***** rmin.c *************************************
 * Description: Calculating the minimum number of
 *   recombination events using the algorithm
 *   in Appendix 2 of Hudson & Kaplan (1985).
 * Reference: Hudson, R. R. and Kaplan, N. L.
 *   (1985). Statistical properties of the number
 *   of recombination events in the history of a
 *   sample of DNA sequences. Genetics, 111:147-
 *   164.
 * Author: Bernhard Haubold, haubold@evolbio.mpg.de
 * Date: Fri Jul 10 10:00:43 2015
 **************************************************/
#include <stdio.h>
#include <stdlib.h>
#include "eprintf.h"
#include "rmin.h"
#include "genDiv.h"

/* numAlleles: number of alleles in sample when considering
 *   positions i and j
 */
short numAlleles(Alignment *a, int i, int j){
  int k, allele[4], x;

  allele[0] = 1;
  for(k=1;k<4;k++)
    allele[k] = 0;
  for(k=1;k<a->m;k++){
    x = 0;
    if(a->al[k][i] != a->al[0][i])
      x += 2;
    if(a->al[k][j] != a->al[0][j])
      x += 1;
    allele[x] = 1;
  }
  x = 1;
  for(k=1;k<4;k++)
    x += allele[x];

  return x;
}

Interval *newInterval(int i, int j){
  Interval *in;

  in = (Interval *)emalloc(sizeof(Interval));
  in->start = i;
  in->end = j;

  return in;
}

int cmpIntervals(const void *a, const void *b){
  
  Interval *ia = *(Interval * const *)a;
  Interval *ib = *(Interval * const *)b;

  return (int)(ia->start - ib->start);
}

int sorted(Interval **interval, int n){
  int i;

  for(i=1;i<n;i++)
    if(interval[i]->start < interval[i-1]->start)
      return 0;

  return 1;
}

int rmin(Alignment *al){
  int i, j, n, end, rm, numInt, c;
  short enclosing;
  Interval **intervals1, **intervals2;

  if(al->poly == NULL)
    findPoly(al);

  n = al->numPoly;
  intervals1 = (Interval **)emalloc(n*(n-1)/2*sizeof(Interval *));
  c = 0;
  for(i=0;i<n-1;i++)
    for(j=i+1;j<n;j++)
      if(numAlleles(al,al->poly[i],al->poly[j]) == 4){
        if(al->poly[i] < al->poly[j])
          intervals1[c++] = newInterval(al->poly[i],al->poly[j]);
        else
          intervals1[c++] = newInterval(al->poly[j],al->poly[i]);
      }
  if(c)
    intervals1 = (Interval **)erealloc(intervals1,c*sizeof(Interval *));
  else
    intervals1 = (Interval **)erealloc(intervals1,sizeof(Interval *));
  intervals2 = (Interval **)emalloc(c*sizeof(Interval *));
  qsort(intervals1, c, sizeof(Interval *), cmpIntervals);
  numInt = 0;
  for(i=0;i<c;i++){
    enclosing = 0;
    for(j=i+1;j<c;j++){
      if(intervals1[i]->start <= intervals1[j]->start && intervals1[i]->end >= intervals1[j]->end){
        enclosing = 1;
        break;
      }
    }    
    if(!enclosing){
      for(j=0;j<numInt;j++)
        if(intervals1[i]->start <= intervals2[j]->start && intervals1[i]->end >= intervals2[j]->end){
          enclosing = 1;
          break;
        }
      if(!enclosing)
        intervals2[numInt++] = intervals1[i];
    }
  }
  if(numInt){
    rm = 1; 
    end = intervals2[0]->end;
  }else
    rm = 0;
  for(i=1;i<numInt;i++){
    if(intervals2[i]->start == intervals2[i-1]->start)
      end = intervals2[i]->end;
    if(intervals2[i]->start >= end){
      rm++;
      end = intervals2[i]->end;
    }
  }
  for(i=0;i<c;i++)
    free(intervals1[i]);
  free(intervals1);
  free(intervals2);
  return rm;
}
